@@ -6,52 +6,54 @@
 #' analysis using \code{ds()}.
 #'
 #' @param data a \code{data.frame} containing at least a column called
-#'        \code{distance}. NOTE! If there is a column called \code{size} in 
+#'        \code{distance}. NOTE! If there is a column called \code{size} in
 #'        the data then it will be interpreted as group/cluster size, see the
-#'        section "Clusters/groups", below.
+#'        section "Clusters/groups", below. One can supply data as a "flat file"
+#'        and not supply \code{region.table}, \code{sample.table} and
+#'        \code{obs.table}, see "Data format", below.
 #' @param truncation either truncation distance (numeric, e.g. 5) or percentage
 #'        (as a string, e.g. "15\%"). Can be supplied as a \code{list}
 #'        with elements \code{left} and \code{right} if left truncation is
-#'        required (e.g. \code{list(left=1,right=20)} or 
-#'        \code{list(left="1\%",right="15\%")} or even 
+#'        required (e.g. \code{list(left=1,right=20)} or
+#'        \code{list(left="1\%",right="15\%")} or even
 #         \code{list(left="1",right="15\%")}).
-#'        When specified as a percentage, the largest \code{right} and 
-#'        smallest \code{left} percent distances are discarded. 
+#'        When specified as a percentage, the largest \code{right} and
+#'        smallest \code{left} percent distances are discarded.
 #' @param transect indicates transect type "line" (default) or "point".
-#' @param formula formula for the scale parameter. For a CDS analysis leave 
+#' @param formula formula for the scale parameter. For a CDS analysis leave
 #'        this as its default \code{~1}.
 #' @param key key function to use; "hn" gives half-normal (default), "hr"
 #'        gives hazard-rate and "unif" gives uniform.
 #' @param adjustment adjustment terms to use; "cos" gives cosine (default),
 #'        "herm" gives Hermite polynomial and "poly" gives simple polynomial.
-#'        "cos" is recommended. A value of \code{NULL} indicates that no 
+#'        "cos" is recommended. A value of \code{NULL} indicates that no
 #'        adjustments are to be fitted.
 #' @param order orders of the adjustment terms to fit (as a vector/scalar), the
-#'        default value (\code{NULL}) will select via AIC. For cosine 
-#'        adjustments, valid orders are integers greater than 2 (except when a 
-#'        uniform key is used, when the minimum order is 1). For Hermite 
-#'        polynomials, even integers equal or greater than 4 are allowed. For 
+#'        default value (\code{NULL}) will select via AIC. For cosine
+#'        adjustments, valid orders are integers greater than 2 (except when a
+#'        uniform key is used, when the minimum order is 1). For Hermite
+#'        polynomials, even integers equal or greater than 4 are allowed. For
 #'        simple polynomials even integers equal or greater than 2 are allowed.
 #' @param scale the scale by which the distances in the adjustment terms are
 #'        divided. Defaults to "width", scaling by the truncation
 #'        distance. If the key is uniform only "width" will be used. The other
 #'        option is "scale": the scale parameter of the detection
-#' @param cutpoints if the data are binned, this vector gives the cutpoints of 
+#' @param cutpoints if the data are binned, this vector gives the cutpoints of
 #'        the bins. Ensure that the first element is 0 (or the left truncation
 #'        distance) and the last is the distance to the end of the furthest bin.
 #'        (Default \code{NULL}, no binning.)
-#'        Note that if \code{data} has columns \code{distbegin} and 
+#'        Note that if \code{data} has columns \code{distbegin} and
 #'        \code{distend} then these will be used as bins if \code{cutpoints}
 #'        is not specified. If both are specified, \code{cutpoints} has
 #'        precedence.
-#' @param monotonicity should the detection function be constrained for 
-#'        monotonicity weakly ("weak"), strictly ("strict") or not at all 
-#'        ("none" or \code{FALSE}). See Montonicity, below. (Default 
+#' @param monotonicity should the detection function be constrained for
+#'        monotonicity weakly ("weak"), strictly ("strict") or not at all
+#'        ("none" or \code{FALSE}). See Montonicity, below. (Default
 #'        \code{FALSE}).
-#' @param region.table \code{data.frame} with two columns: 
+#' @param region.table \code{data.frame} with two columns:
 #'        \tabular{ll}{ \code{Region.Label} \tab label for the region\cr
-#'                     \code{Area} \tab area of the region\cr} 
-#'        \code{region.table} has one row for each stratum. If there is no 
+#'                     \code{Area} \tab area of the region\cr}
+#'        \code{region.table} has one row for each stratum. If there is no
 #'        stratification then \code{region.table} has one entry with \code{Area}
 #'        corresponding to the total survey area.
 #' @param sample.table \code{data.frame} mapping the regions to the samples (
@@ -59,28 +61,30 @@
 #'        \tabular{ll}{\code{Sample.Label} \tab label for the sample\cr
 #'                     \code{Region.Label} \tab label for the region that the
 #'                          sample belongs to.\cr
-#'                     \code{Effort} \tab the effort expended in that sample 
-#'                          (e.g. transect length).\cr} 
-#' @param obs.table \code{data.frame} mapping the individual observations 
+#'                     \code{Effort} \tab the effort expended in that sample
+#'                          (e.g. transect length).\cr}
+#' @param obs.table \code{data.frame} mapping the individual observations
 #'        (objects) to regions and samples. There should be three columns:
 #'        \tabular{ll}{\code{object} \tab \cr
 #'                     \code{Region.Label} \tab label for the region that the
 #'                          sample belongs to.\cr
 #'                     \code{Sample.Label} \tab label for the sample\cr}
-#' @param convert.units conversion between units for abundance estimation, 
-#'        see "Units", below. (Defaults to 1, implying all of the units are 
+#' @param convert.units conversion between units for abundance estimation,
+#'        see "Units", below. (Defaults to 1, implying all of the units are
 #'        "correct" already.)
 #'
-#' @param method optimization method to use (any method usable by 
-#'        \code{\link{optim}} or \code{\link{optimx}}). Defaults to 
+#' @param method optimization method to use (any method usable by
+#'        \code{\link{optim}} or \code{\link{optimx}}). Defaults to
 #'        "nlminb".
 #'
 #' @param debug.level print debugging output. 0=none, 1-3 increasing level of
 #'        debugging output.
 #'
-#' @param quiet surpress non-warning messages (useful for bootstraps etc).
+#' @param quiet surpress non-essential messages (useful for bootstraps etc).
 #'              Default value FALSE.
 #'
+#' @param initial.values a \code{list} of named starting values, see
+#'        \code{\link{ddf}}. Only allowed when AIC term selection is not used.
 #'
 #' @return a list with elements:
 #'        \tabular{ll}{\code{ddf} \tab a detection function model object.\cr
@@ -93,11 +97,11 @@
 #'  \code{sample.table} and \code{obs.table} must be supplied.
 #'
 #' @section Clusters/groups:
-#'  Note that if the data contains a column named \code{size} and 
-#'  \code{region.table}, \code{sample.table} and \code{obs.table} are supplied, 
-#'   cluster size will be estimated and density/abundance will be based on a 
-#'  clustered analsis of the data. Setting this column to be \code{NULL} will 
-#'  perform a non-clustred analysis (for example if "size" means something else 
+#'  Note that if the data contains a column named \code{size} and
+#'  \code{region.table}, \code{sample.table} and \code{obs.table} are supplied,
+#'   cluster size will be estimated and density/abundance will be based on a
+#'  clustered analsis of the data. Setting this column to be \code{NULL} will
+#'  perform a non-clustred analysis (for example if "size" means something else
 #'  if your dataset).
 #'
 # THIS IS STOLEN FROM mrds, sorry Jeff!
@@ -105,42 +109,57 @@
 #'  In extrapolating to the entire survey region it is important that
 #'  the unit measurements be consistent or converted for consistency.
 #'  A conversion factor can be specified with the \code{convert.units}
-#'  variable.  The values of \code{Area} in \code{region.table}, must be made 
-#'  consistent with the units for \code{Effort} in \code{sample.table} and the 
+#'  variable.  The values of \code{Area} in \code{region.table}, must be made
+#'  consistent with the units for \code{Effort} in \code{sample.table} and the
 #'  units of \code{distance} in the \code{data.frame} that was analyzed.  It is
-#'  easiest if the units of \code{Area} are the square of the units of 
-#'  \code{Effort} and then it is only necessary to convert the units of 
+#'  easiest if the units of \code{Area} are the square of the units of
+#'  \code{Effort} and then it is only necessary to convert the units of
 #'  \code{distance} to the units of \code{Effort}. For example, if \code{Effort}
-#'   was entered in kilometers and \code{Area} in square kilometers and 
-#'  \code{distance} in meters then using \code{convert.units=0.001} would 
-#'  convert meters to kilometers, density would be expressed in square 
-#'  kilometers which would then be consistent with units for \code{Area}.  
-#'  However, they can all be in different units as long as the appropriate 
-#'  composite value for \code{convert.units} is chosen.  Abundance for a survey 
-#'  region can be expressed as: \code{A*N/a} where \code{A} is \code{Area} for 
-#'  the survey region, \code{N} is the abundance in the covered (sampled) 
-#'  region, and \code{a} is the area of the sampled region and is in units of 
-#'  \code{Effort * distance}.  The sampled region \code{a} is multiplied by 
-#'   \code{convert.units}, so it should be chosen such that the result is in 
-#'  the same units as \code{Area}.  For example, if \code{Effort} was entered 
-#'  in kilometers, \code{Area} in hectares (100m x 100m) and \code{distance} 
-#'  in meters, then using \code{convert.units=10} will convert \code{a} to 
-#'  units of hectares (100 to convert meters to 100 meters for distance and 
+#'   was entered in kilometers and \code{Area} in square kilometers and
+#'  \code{distance} in meters then using \code{convert.units=0.001} would
+#'  convert meters to kilometers, density would be expressed in square
+#'  kilometers which would then be consistent with units for \code{Area}.
+#'  However, they can all be in different units as long as the appropriate
+#'  composite value for \code{convert.units} is chosen.  Abundance for a survey
+#'  region can be expressed as: \code{A*N/a} where \code{A} is \code{Area} for
+#'  the survey region, \code{N} is the abundance in the covered (sampled)
+#'  region, and \code{a} is the area of the sampled region and is in units of
+#'  \code{Effort * distance}.  The sampled region \code{a} is multiplied by
+#'   \code{convert.units}, so it should be chosen such that the result is in
+#'  the same units as \code{Area}.  For example, if \code{Effort} was entered
+#'  in kilometers, \code{Area} in hectares (100m x 100m) and \code{distance}
+#'  in meters, then using \code{convert.units=10} will convert \code{a} to
+#'  units of hectares (100 to convert meters to 100 meters for distance and
 #'  .1 to convert km to 100m units).
 #'
 #'  @section Monotonicity: When adjustment terms are used, it is possible for
 #'  the detection function to not always decrease with increasing distance.
 #'  This is unrealistic and can lead to bias. To avoid this, the detection
-#'  function can be constrained for monotonicity. 
+#'  function can be constrained for monotonicity.
 #'
-#'  Monotonicity constraints are supported in a similar way to that described 
-#'  in Buckland et al (2001). 20 equally spaced points over the range of the 
-#'  detection function (left to right truncation) are evaluated at each round 
-#'  of the optimisation and the function is constrained to be either always 
-#'  less than it's value at zero (\code{"weak"}) or such that each value is 
-#'  less than or equal to the previous point (monotonically decreasing; 
+#'  Monotonicity constraints are supported in a similar way to that described
+#'  in Buckland et al (2001). 20 equally spaced points over the range of the
+#'  detection function (left to right truncation) are evaluated at each round
+#'  of the optimisation and the function is constrained to be either always
+#'  less than it's value at zero (\code{"weak"}) or such that each value is
+#'  less than or equal to the previous point (monotonically decreasing;
 #'  \code{"strict"}).
-#'   
+#'
+#'  @section Binning: Note that binning is performed such that bin 1 is all
+#'  distances greater or equal to cutpoint 1 (>=0 or left truncation distance)
+#'  and less than cutpoint 2. Bin 2 is then distances greater or equal to
+#'  cutpoint 2 and less than cutpoint 3 and so on.
+#'
+#'  @section Data format: One can supply \code{data} only to simply fit a
+#'  detection function. However, if abundance/density estimates are necessary
+#'  further information is required. Either the \code{region.table},
+#'  \code{sample.table} and \code{obs.table} \code{data.frame}s can be supplied
+#'  or all data can be supplied as a "flat file" in the \code{data} argument. In
+#'  this format each row in data has additional information that would
+#'  ordinarily be in the other tables. This usually means that there are
+#'  additional columns named: \code{Sample.Label}, \code{Region.Label},
+#'  \code{Effort} and \code{Area} for each observation.
+#'
 #' @author David L. Miller
 #' @export
 #'
@@ -151,86 +170,70 @@
 #'
 #' @examples
 #'
-#'  # An example from mrds, the golf tee data.
-#'  library(Distance)
-#'  data(book.tee.data)
-#'  tee.data<-book.tee.data$book.tee.dataframe[book.tee.data$book.tee.dataframe$observer==1,]
-#'  ds.model<-ds(tee.data,4,monotonicity="strict")
-#'  summary(ds.model)
-#'  plot(ds.model)
+#' # An example from mrds, the golf tee data.
+#' library(Distance)
+#' data(book.tee.data)
+#' tee.data<-book.tee.data$book.tee.dataframe[book.tee.data$book.tee.dataframe$observer==1,]
+#' ds.model<-ds(tee.data,4,monotonicity="strict")
+#' summary(ds.model)
+#' plot(ds.model)
 #'
-#'  # same model, but calculating abundance
-#'  # need to supply the region, sample and observation tables
-#'  region<-book.tee.data$book.tee.region
-#'  samples<-book.tee.data$book.tee.samples
-#'  obs<-book.tee.data$book.tee.obs
-#' 
-#'  ds.dht.model<-ds(tee.data,4,region.table=region,monotonicity="strict",
-#'               sample.table=samples,obs.table=obs)
-#'  summary(ds.dht.model)
+#' \dontrun{
+#' # same model, but calculating abundance
+#' # need to supply the region, sample and observation tables
+#' region<-book.tee.data$book.tee.region
+#' samples<-book.tee.data$book.tee.samples
+#' obs<-book.tee.data$book.tee.obs
 #'
-#'  # specify order 2 cosine adjustments
-#'  ds.model.cos2<-ds(tee.data,4,adjustment="cos",order=2)
-#'  summary(ds.model.cos2)
+#' ds.dht.model<-ds(tee.data,4,region.table=region,monotonicity="strict",
+#'              sample.table=samples,obs.table=obs)
+#' summary(ds.dht.model)
 #'
-#'  # specify order 2 and 3 cosine adjustments - LOTS of non-monotonicity!
-#'  ds.model.cos24<-ds(tee.data,4,adjustment="cos",order=c(2,3))
-#'  summary(ds.model.cos24)
+#' # specify order 2 cosine adjustments
+#' ds.model.cos2<-ds(tee.data,4,adjustment="cos",order=2)
+#' summary(ds.model.cos2)
 #'
-#'  # truncate the largest 10% of the data and fit only a hazard-rate
-#'  # detection function
-#'  ds.model.hr.trunc<-ds(tee.data,truncation="10%",key="hr",adjustment=NULL)
-#'  summary(ds.model.hr.trunc)
+#' # specify order 2 and 3 cosine adjustments - LOTS of non-monotonicity!
+#' ds.model.cos24<-ds(tee.data,4,adjustment="cos",order=c(2,3))
+#' summary(ds.model.cos24)
+#'
+#' # truncate the largest 10% of the data and fit only a hazard-rate
+#' # detection function
+#' ds.model.hr.trunc<-ds(tee.data,truncation="10%",key="hr",adjustment=NULL)
+#' summary(ds.model.hr.trunc)
+#'}
 #'
 ds<-function(data, truncation=NULL, transect="line", formula=~1, key="hn",
              adjustment="cos", order=NULL, scale="width", cutpoints=NULL,
              monotonicity=FALSE,
              region.table=NULL,sample.table=NULL,obs.table=NULL,
-             convert.units=1,method="nlminb",quiet=FALSE,debug.level=0){
+             convert.units=1,method="nlminb",quiet=FALSE,debug.level=0,
+             initial.values=NULL){
 
   # this routine just creates a call to mrds, it's not very exciting
   # or fancy, it does do a lot of error checking though
 
-  ## make sure that the data are in the right format first
-  if(is.null(data$distance)){
-    stop("Your data must have a column called 'distance'!")
-  }
-
-  # make sure that we have a data.frame()
-  data<-data.frame(data)
-
-  # first see if the data has detected/observer/object columns, if not add them
-  if(!any(names(data)=="observer")){
-    data<-cbind(data,observer=rep(1,nrow(data)))
-  }
-  if(!any(names(data)=="detected")){
-    data<-cbind(data,detected=rep(1,nrow(data)))
-  }
-  if(!any(names(data)=="object")){
-    data<-cbind(data,object=1:nrow(data))
-  }
-
-  # check that dht info has the right column titles
-  if(!is.null(region.table) & !is.null(sample.table) & !is.null(obs.table)){
-    if(!all(c("Region.Label","Area") %in% names(region.table))){
-      stop("region.table must have columns named 'Region.Label' and 'Area'")
-    }
-    if(!all(c("Region.Label","Sample.Label","Effort") %in% names(sample.table))){
-      stop("sample.table must have columns named 'Region.Label', 'Sample.Label' and 'Effort'")
-    }
-    if(!all(c("Region.Label","Sample.Label","object") %in% names(obs.table))){
-      stop("obs.table must have columns names 'Region.Label', 'Sample.Label' and 'object'")
-    }
-  }
+  # check that the data is okay
+  data <- checkdata(data,region.table,sample.table,obs.table)
+  region.table <- data$region.table
+  sample.table <- data$sample.table
+  obs.table    <- data$obs.table
+  data         <- data$data
 
   # truncation
   if(is.null(truncation)){
     stop("Please supply truncation distance or percentage.")
+  }else if(any(unlist(lapply(truncation,is.character))) &
+           (!is.null(cutpoints) |
+            any(c("distbegin","distend") %in% colnames(data))
+           )){
+    stop("Truncation cannot be supplied as a percentage with binned data")
   }else{
     # if we have left truncation too...
     if(is.list(truncation)){
-      if((any(names(truncation)=="left") & any(names(truncation)=="right")) &
-          length(truncation)==2){
+      if((any(names(truncation)=="left") &
+          any(names(truncation)=="right")) &
+         length(truncation)==2){
 
         # check for each of left and right that we have % or distance...
         # left
@@ -316,7 +319,7 @@ ds<-function(data, truncation=NULL, transect="line", formula=~1, key="hn",
       aic.search <- FALSE
       if(any(order != ceiling(order))){
           stop("Adjustment orders must be integers.")
-      } 
+      }
 
       if(formula != ~1){
         stop("Cannot use both adjustments and covariates, choose one!")
@@ -339,6 +342,7 @@ ds<-function(data, truncation=NULL, transect="line", formula=~1, key="hn",
       # if there are covariates then don't do the AIC search
       if(formula != ~1){
         aic.search <- FALSE
+        message("Cannot perfrom AIC adjustment term selection when covariates are used.")
       }else{
       # otherwise go ahead and set up the candidate adjustment orders
         aic.search <- TRUE
@@ -378,7 +382,7 @@ ds<-function(data, truncation=NULL, transect="line", formula=~1, key="hn",
   ### binning
   if(is.null(cutpoints)){
     if(any(names(data)=="distend") & any(names(data)=="distbegin")){
-      warning("No cutpoints specified but distbegin and distend are columns in data. Performing a binned analysis...")
+      message("No cutpoints specified but distbegin and distend are columns in data. Performing a binned analysis...")
       binned <- TRUE
       breaks <- sort(unique(c(data$distend,data$distbegin)))
     }else{
@@ -397,7 +401,7 @@ ds<-function(data, truncation=NULL, transect="line", formula=~1, key="hn",
 
     # remove distbegin and distend if they already exist
     if(any(names(data)=="distend") & any(names(data)=="distbegin")){
-      warning("data already has distend and distbegin columns, removing them and appling binning as specified by cutpoints.")
+      message("data already has distend and distbegin columns, removing them and appling binning as specified by cutpoints.")
       data$distend<-NULL
       data$distbegin<-NULL
     }
@@ -426,15 +430,21 @@ ds<-function(data, truncation=NULL, transect="line", formula=~1, key="hn",
     stop("monotonicity must be one of \"none\", FALSE, \"weak\" or \"strict\".")
   }
 
-  # can't do monotonicity and covariates, turn that off!
+  # can't do monotonicity and covariates, fail!
   if(mono & formula!=as.formula("~1")){
-    warning("Monotonicity cannot be enforced with covariates.")
-    mono <- mono.strict <- FALSE
+    stop("Monotonicity cannot be enforced with covariates.")
   }
 
   # set up the control options
   control <- list(optimx.method=method, showit=debug.level)
 
+  # if initial values were supplied, pass them on
+  if(!is.null(initial.values) & !aic.search){
+    control$initial <- initial.values
+  }else if(!is.null(initial.values) & aic.search){
+    stop("Cannot supply initial values when using AIC term selection")
+  }
+
   ### Actually fit some models here
 
   # construct the meta data object...
@@ -474,7 +484,7 @@ ds<-function(data, truncation=NULL, transect="line", formula=~1, key="hn",
     # MCDS model
     }else{
       model.formula <- paste("~mcds(key = \"",key,"\",",
-                                  "formula =~",as.character(formula)[2],sep="") 
+                                  "formula =~",as.character(formula)[2],sep="")
     }
 
     # build a message to let the user know what is being fitted
@@ -544,8 +554,16 @@ ds<-function(data, truncation=NULL, transect="line", formula=~1, key="hn",
         message("  Model failed to converge.")
       }
     }else{
-      message(paste0("\n\nError in model fitting, returning: ",model$name.message))
-      model <- last.model
+      if(last.model$criterion == Inf & length(last.model)==1){
+        message("\n\nAll models failed to fit!\n")
+        model <- NULL
+      }else{
+        message(paste0("\n\nError in model fitting, returning: ",
+                       sub("^Fitting ","",this.message)))
+        message(paste0("\n  Error: ",model[1],"\n"))
+        model <- NULL
+        model <- last.model
+      }
       break
     }
   }
